# coding: utf-8

# flake8: noqa

"""
    HPE Machine Learning Inference Software (MLIS/Aioli)

    HPE MLIS is *Aioli* -- The AI On-line Inference Platform that enables easy deployment, tracking, and serving of your packaged models regardless of your preferred AI framework.

    The version of the OpenAPI document: 1.3.0-dev21
    Contact: community@determined-ai
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from aiolirest.api.authentication_api import AuthenticationApi
from aiolirest.api.deployments_api import DeploymentsApi
from aiolirest.api.information_api import InformationApi
from aiolirest.api.packaged_models_api import PackagedModelsApi
from aiolirest.api.projects_api import ProjectsApi
from aiolirest.api.registries_api import RegistriesApi
from aiolirest.api.roles_api import RolesApi
from aiolirest.api.templates_api import TemplatesApi
from aiolirest.api.tokens_api import TokensApi
from aiolirest.api.users_api import UsersApi

# import ApiClient
from aiolirest.api_response import ApiResponse
from aiolirest.api_client import ApiClient
from aiolirest.configuration import Configuration
from aiolirest.exceptions import OpenApiException
from aiolirest.exceptions import ApiTypeError
from aiolirest.exceptions import ApiValueError
from aiolirest.exceptions import ApiKeyError
from aiolirest.exceptions import ApiAttributeError
from aiolirest.exceptions import ApiException

# import models into sdk package
from aiolirest.models.auto_scaling_template import AutoScalingTemplate
from aiolirest.models.auto_scaling_template_request import AutoScalingTemplateRequest
from aiolirest.models.autoscaling import Autoscaling
from aiolirest.models.configuration_resources import ConfigurationResources
from aiolirest.models.controller_config import ControllerConfig
from aiolirest.models.controller_status import ControllerStatus
from aiolirest.models.deployment import Deployment
from aiolirest.models.deployment_model_version import DeploymentModelVersion
from aiolirest.models.deployment_priority import DeploymentPriority
from aiolirest.models.deployment_request import DeploymentRequest
from aiolirest.models.deployment_state import DeploymentState
from aiolirest.models.deployment_token import DeploymentToken
from aiolirest.models.deployment_token_patch_request import DeploymentTokenPatchRequest
from aiolirest.models.deployment_token_request import DeploymentTokenRequest
from aiolirest.models.ezua_info import EZUAInfo
from aiolirest.models.error_response import ErrorResponse
from aiolirest.models.event_info import EventInfo
from aiolirest.models.failure_info import FailureInfo
from aiolirest.models.login_request import LoginRequest
from aiolirest.models.login_response import LoginResponse
from aiolirest.models.model_auth_token import ModelAuthToken
from aiolirest.models.model_response import ModelResponse
from aiolirest.models.observability import Observability
from aiolirest.models.packaged_model import PackagedModel
from aiolirest.models.packaged_model_request import PackagedModelRequest
from aiolirest.models.project import Project
from aiolirest.models.project_request import ProjectRequest
from aiolirest.models.resource_profile import ResourceProfile
from aiolirest.models.resources_template import ResourcesTemplate
from aiolirest.models.resources_template_request import ResourcesTemplateRequest
from aiolirest.models.role import Role
from aiolirest.models.role_assignment import RoleAssignment
from aiolirest.models.role_assignments import RoleAssignments
from aiolirest.models.security import Security
from aiolirest.models.success_response import SuccessResponse
from aiolirest.models.telemetry_info import TelemetryInfo
from aiolirest.models.trained_model_registry import TrainedModelRegistry
from aiolirest.models.trained_model_registry_request import TrainedModelRegistryRequest
from aiolirest.models.user import User
from aiolirest.models.user_patch_request import UserPatchRequest
from aiolirest.models.user_request import UserRequest
from aiolirest.models.user_session import UserSession
